@@ -25,16 +25,14 @@
 
 #include "stdc.h"
 
-#if !defined (SIG_DFL)
-#  include <signal.h>		/* for sig_atomic_t */
-#endif
+#include <signal.h>		/* for sig_atomic_t */
 
 #if !defined (SIGABRT) && defined (SIGIOT)
 #  define SIGABRT SIGIOT
 #endif
 
 #define sighandler RETSIGTYPE
-typedef RETSIGTYPE SigHandler __P((int));
+typedef RETSIGTYPE SigHandler PARAMS((int));
 
 #if defined (VOID_SIGHANDLER)
 #  define SIGRETURN(n)	return
@@ -48,7 +46,7 @@ typedef RETSIGTYPE SigHandler __P((int));
 #if !defined (HAVE_POSIX_SIGNALS)
 #  define set_signal_handler(sig, handler) (SigHandler *)signal (sig, handler)
 #else
-extern SigHandler *set_signal_handler __P((int, SigHandler *));	/* in sig.c */
+extern SigHandler *set_signal_handler PARAMS((int, SigHandler *));	/* in sig.c */
 #endif /* _POSIX_VERSION */
 
 #if !defined (SIGCHLD) && defined (SIGCLD)
@@ -115,25 +113,28 @@ extern int interrupt_immediately;	/* no longer used */
 extern int terminate_immediately;
 
 /* Functions from sig.c. */
-extern sighandler termsig_sighandler __P((int));
-extern void termsig_handler __P((int));
-extern sighandler sigint_sighandler __P((int));
-extern void initialize_signals __P((int));
-extern void initialize_terminating_signals __P((void));
-extern void reset_terminating_signals __P((void));
-extern void top_level_cleanup __P((void));
-extern void throw_to_top_level __P((void));
-extern void jump_to_top_level __P((int)) __attribute__((__noreturn__));
+extern sighandler termsig_sighandler PARAMS((int));
+extern void termsig_handler PARAMS((int));
+extern sighandler sigint_sighandler PARAMS((int));
+extern void initialize_signals PARAMS((int));
+extern void initialize_terminating_signals PARAMS((void));
+extern void reset_terminating_signals PARAMS((void));
+extern void top_level_cleanup PARAMS((void));
+extern void throw_to_top_level PARAMS((void));
+extern void jump_to_top_level PARAMS((int)) __attribute__((__noreturn__));
+extern void restore_sigmask PARAMS((void));
 
-extern sighandler sigwinch_sighandler __P((int));
-extern void set_sigwinch_handler __P((void));
-extern void unset_sigwinch_handler __P((void));
+extern sighandler sigwinch_sighandler PARAMS((int));
+extern void set_sigwinch_handler PARAMS((void));
+extern void unset_sigwinch_handler PARAMS((void));
 
-extern sighandler sigterm_sighandler __P((int));
+extern sighandler sigterm_sighandler PARAMS((int));
 
 /* Functions defined in trap.c. */
-extern SigHandler *set_sigint_handler __P((void));
-extern SigHandler *trap_to_sighandler __P((int));
-extern sighandler trap_handler __P((int));
+extern SigHandler *set_sigint_handler PARAMS((void));
+extern SigHandler *trap_to_sighandler PARAMS((int));
+extern sighandler trap_handler PARAMS((int));
 
+extern int block_trapped_signals PARAMS((sigset_t *, sigset_t *));
+extern int unblock_trapped_signals PARAMS((sigset_t *));
 #endif /* _SIG_H_ */
